/*
    Yin 3D Game Engine
    Copyright (C) 2020 Mark Sowden <hogsy@oldtimes-software.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
/* Copyright (C) 2020 Mark Sowden <markelswo@gmail.com>
 * Project Yin
 * */

#pragma once

typedef enum ActorType {
	ACTOR_NONE,
	ACTOR_PLAYER,

	ACTOR_TRIGGER_VOLUME,

	MAX_ACTOR_TYPES
} ActorType;

typedef enum ActorMovementType {
	ACTOR_MOVEMENT_CUSTOM,
	ACTOR_MOVEMENT_PHYSICS,

	MAX_ACTOR_MOVEMENT_TYPES
} ActorMovementType;

typedef enum ActorCollisionGroup {
	PL_BITFLAG( ACTOR_COLLISION_GROUP_WORLD, 0 ),
	PL_BITFLAG( ACTOR_COLLISION_GROUP_PLAYER, 1 ),
	PL_BITFLAG( ACTOR_COLLISION_GROUP_MONSTER, 2 ),
} ActorCollisionGroup;

typedef struct Actor Actor;

void Act_Initialize( void );
void Act_Shutdown( void );

void Act_SpawnActors( void );
void Act_DrawActors( void );
void Act_TickActors( void );

Actor *Act_SpawnActor( ActorType type, PLVector3 position, float angle );
Actor *Act_DestroyActor( Actor *self );

ActorType Act_GetType( const Actor *self );

void Act_SetPosition( Actor *self, const PLVector3 *position );
PLVector3 Act_GetPosition( const Actor *self );

void Act_SetVelocity( Actor *self, const PLVector3 *velocity );
PLVector3 Act_GetVelocity( const Actor *self );

void Act_SetAngle( Actor *self, float angle );
float Act_GetAngle( const Actor *self );

void Act_SetViewPitch( Actor *self, float viewPitch );
float Act_GetViewPitch( const Actor *self );

void Act_SetUserData( Actor *self, void *userData );
void *Act_GetUserData( Actor *self );

void Act_SetCurrentFrame( Actor *self, unsigned int frame );
unsigned int Act_GetCurrentFrame( const Actor *self );

void Act_SetViewOffset( Actor *self, float viewOffset );
float Act_GetViewOffset( Actor *self );

void Act_SetBounds( Actor *self, PLVector3 mins, PLVector3 maxs );
const PLCollisionAABB *Act_GetBounds( Actor *self );

PLVector3 Act_GetForward( const Actor *self );

/* generic monster functions */
void Monster_Collide( struct Actor *self, struct Actor *other, void *userData );

/* player functions */
struct GfxCamera *Player_GetCamera( Actor *self );
bool Player_IsPointVisible( Actor *self, const PLVector2 *point );
